@@ -19,12 +19,15 @@ class Daemonize(object):
     - pid: path to the pidfile.
     - action: your custom function which will be executed after daemonization.
     - keep_fds: optional list of fds which should not be closed.
+    - auto_close_fds: optional parameter to not close opened fds.
     - privileged_action: action that will be executed before drop privileges if user or
                          group parameter is provided.
     - user: drop privileges to this user if provided.
     - group: drop privileges to this group if provided.
+    - verbose: send debug messages to logger if provided.
+    - logger: use this logger object instead of creating new one, if provided.
     """
-    def __init__(self, app, pid, action, keep_fds=None, privileged_action=None, user=None, group=None, verbose=False):
+    def __init__(self, app, pid, action, keep_fds=None, auto_close_fds=True, privileged_action=None, user=None, group=None, verbose=False, logger=None):
         self.app = app
         self.pid = pid
         self.action = action
@@ -32,12 +35,9 @@ class Daemonize(object):
         self.privileged_action = privileged_action or (lambda: ())
         self.user = user
         self.group = group
-        # Initialize logging.
-        self.logger = logging.getLogger(self.app)
-        self.logger.setLevel(logging.DEBUG)
-        # Display log messages only on defined handlers.
-        self.logger.propagate = False
+        self.logger = logger
         self.verbose = verbose
+        self.auto_close_fds = auto_close_fds
 
     def sigterm(self, signum, frame):
         """ sigterm method
@@ -59,6 +59,28 @@ class Daemonize(object):
         """ start method
         Main daemonization process.
         """
+        # If pidfile already exists, we should read pid from there; to overwrite it, if locking
+        # will fail, because locking attempt somehow purges the file contents.
+        if os.path.isfile(self.pid):
+            with open(self.pid, "r") as old_pidfile:
+                old_pid = old_pidfile.read()
+        # Create a lockfile so that only one instance of this daemon is running at any time.
+        try:
+            lockfile = open(self.pid, "w")
+        except IOError:
+            print("Unable to create the pidfile.")
+            sys.exit(1)
+        try:
+            # Try to get an exclusive lock on the file. This will fail if another process has the file
+            # locked.
+            fcntl.flock(lockfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
+        except IOError:
+            print("Unable to lock on the pidfile.")
+            # We need to overwrite the pidfile if we got here.
+            with open(self.pid, "w") as pidfile:
+                pidfile.write(old_pid)
+            sys.exit(1)
+
         # Fork, creating a new process for the child.
         process_id = os.fork()
         if process_id < 0:
@@ -78,6 +100,9 @@ class Daemonize(object):
             # Uh oh, there was a problem.
             sys.exit(1)
 
+        # Add lockfile to self.keep_fds.
+        self.keep_fds.append(lockfile.fileno())
+
         # Close all file descriptors, except the ones mentioned in self.keep_fds.
         devnull = "/dev/null"
         if hasattr(os, "devnull"):
@@ -85,35 +110,48 @@ class Daemonize(object):
             # than /dev/null.
             devnull = os.devnull
 
-        for fd in range(resource.getrlimit(resource.RLIMIT_NOFILE)[0]):
-            if fd not in self.keep_fds:
-                try:
-                    os.close(fd)
-                except OSError:
-                    pass
+        if self.auto_close_fds:
+            for fd in range(3, resource.getrlimit(resource.RLIMIT_NOFILE)[0]):
+                if fd not in self.keep_fds:
+                    try:
+                        os.close(fd)
+                    except OSError:
+                        pass
 
-        os.open(devnull, os.O_RDWR)
-        os.dup(0)
-        os.dup(0)
+        devnull_fd = os.open(devnull, os.O_RDWR)
+        os.dup2(devnull_fd, 0)
+        os.dup2(devnull_fd, 1)
+        os.dup2(devnull_fd, 2)
 
-        # Initialize syslog.
-        # It will work on OS X and Linux. No FreeBSD support, guys, I don't want to import re here
-        # to parse your peculiar platform string.
-        if sys.platform == "darwin":
-            syslog_address = "/var/run/syslog"
-        else:
-            syslog_address = "/dev/log"
-        syslog = handlers.SysLogHandler(syslog_address)
-        if self.verbose:
-            syslog.setLevel(logging.DEBUG)
-        else:
-            syslog.setLevel(logging.INFO)
-        # Try to mimic to normal syslog messages.
-        formatter = logging.Formatter("%(asctime)s %(name)s: %(message)s",
-                                      "%b %e %H:%M:%S")
-        syslog.setFormatter(formatter)
+        if self.logger is None:
+            # Initialize logging.
+            self.logger = logging.getLogger(self.app)
+            self.logger.setLevel(logging.DEBUG)
+            # Display log messages only on defined handlers.
+            self.logger.propagate = False
 
-        self.logger.addHandler(syslog)
+            # Initialize syslog.
+            # It will work on OS X and Linux. No FreeBSD support, guys, I don't want to import re here
+            # to parse your peculiar platform string.
+            if sys.platform == "darwin":
+                syslog_address = "/var/run/syslog"
+            else:
+                syslog_address = "/dev/log"
+
+            # We will continue with syslog initialization only if actually have such capabilities
+            # on the machine we are running this.
+            if os.path.isfile(syslog_address):
+                syslog = handlers.SysLogHandler(syslog_address)
+                if self.verbose:
+                    syslog.setLevel(logging.DEBUG)
+                else:
+                    syslog.setLevel(logging.INFO)
+                # Try to mimic to normal syslog messages.
+                formatter = logging.Formatter("%(asctime)s %(name)s: %(message)s",
+                                              "%b %e %H:%M:%S")
+                syslog.setFormatter(formatter)
+
+                self.logger.addHandler(syslog)
 
         # Set umask to default to safe file permissions when running as a root daemon. 027 is an
         # octal number which we are typing as 0o27 for Python3 compatibility.
@@ -153,21 +191,11 @@ class Daemonize(object):
                 sys.exit(1)
 
         try:
-            # Create a lockfile so that only one instance of this daemon is running at any time.
-            lockfile = open(self.pid, "w")
-        except IOError:
-            self.logger.error("Unable to create a pidfile.")
-            sys.exit(1)
-        try:
-            # Try to get an exclusive lock on the file. This will fail if another process has the file
-            # locked.
-            fcntl.lockf(lockfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            # Record the process id to the lockfile. This is standard practice for daemons.
             lockfile.write("%s" % (os.getpid()))
             lockfile.flush()
         except IOError:
-            self.logger.error("Unable to lock on the pidfile.")
-            os.remove(self.pid)
+            self.logger.error("Unable to write pid to the pidfile.")
+            print("Unable to write pid to the pidfile.")
             sys.exit(1)
 
         # Set custom action on SIGTERM.
